import 'dart:io';
import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:uuid/uuid.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxwidgets/XF_MAIN/XApp.dart';
import 'package:xfxwidgets/xMediaItem.dart';
import 'package:xfxwidgets/xfxwidgets.dart';
import 'package:xtoolspackage/XLibStorage.dart';
import 'package:xtoolspackage/xtoolspackage.dart';
import 'package:audio_waveforms/audio_waveforms.dart';
import 'package:video_thumbnail/video_thumbnail.dart' as thumb;
import 'package:multiple_image_camera/camera_file.dart';
import 'package:multiple_image_camera/multiple_image_camera.dart';

class XMediaRecorder<T> extends StatefulWidget {
  //Ci facciamo passare l'oggetto da visualizzare T
  XMediaRecorder(
    Key? key, {
    required this.itemOriginal,
    required this.parentSetState,
    this.onTakeMedia,
    this.onStartTakeVideo,
    this.onEndTakeVideo,
    // required this.onTake_Video,
    // required this.onTake_Audio,
    required this.xApp,
    required this.lXMedias,
    this.title,
    this.title_BackColor,
    this.title_Color,
  });

  String? title;
  Color? title_BackColor;
  Color? title_Color;

  List<XMediaItem> lXMedias = List<XMediaItem>.empty(growable: true);
  T itemOriginal;
  void Function(void Function() p1) parentSetState;
  Function(void Function(void Function() p1))? onTakeMedia;
  Function(void Function(void Function() p1))? onStartTakeVideo;
  Function(void Function(void Function() p1))? onEndTakeVideo;
  // Function(XFile, T, bool, void Function(void Function())) onTake_Video;
  // Function(String, T, bool, void Function(void Function())) onTake_Audio;
  XApp xApp;
  @override
  State<XMediaRecorder<T>> createState() => XMediaRecorderState<T>();
}

class XMediaRecorderState<T> extends State<XMediaRecorder<T>> with WidgetsBindingObserver, TickerProviderStateMixin {
  late CameraController? photoController;
  int pointers = 0;
  static String? urlRTSP;
  XFile? imageFile;
  XFile? videoFile;
  double _minAvailableZoom = 1.0;
  double _maxAvailableZoom = 1.0;
  double _currentScale = 1.0;
  double _baseScale = 1.0;
  // ignore: unused_field
  double _minAvailableExposureOffset = 0.0;
  // ignore: unused_field
  double _maxAvailableExposureOffset = 0.0;
  bool outCamera = false;

  String waiterMessage = "";

  TextEditingController? urlRTSPcontroller = TextEditingController();
  late Orientation? orientation;

  bool? createAudio = false;
  File? xFileUI;

  late final RecorderController recorderController;

  String? musicFile;
  bool isRecording_Audio = false;
  bool isRecordingCompleted = false;
  bool isLoading = true;

  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    XStatefulWidgetBase.color_Waiter = Color.fromARGB(255, 22, 228, 255);
    loadPlayer();
    if (urlRTSP == null) {
      urlRTSPcontroller!.text = "rtsp://";
    } else {
      urlRTSPcontroller!.text = urlRTSP ?? "";
    }
    super.initState();
    _initialiseControllers();
  }

  loadPlayer() async {
    await onNewCameraSelected();
    setFlashMode(FlashMode.off, setState);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    photoController!.dispose();
    urlRTSPcontroller!.dispose();
    // player.dispose();
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    // App state changed before we got the chance to initialize.
    if (photoController == null || !photoController!.value.isInitialized) {
      return;
    }

    if (state == AppLifecycleState.inactive) {
      // Free up memory when camera not active
      photoController!.dispose();
    } else if (state == AppLifecycleState.resumed) {
      // Reinitialize the camera with same properties
      onNewCameraSelected();
    }
  }

  void showInSnackBar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text(message)));
  }

  Future<void> onNewCameraSelected() async {
    photoController = CameraController(widget.xApp.state.cameras[0], ResolutionPreset.max, enableAudio: true);
    photoController!.setFocusMode(FocusMode.locked);
    photoController!.addListener(() {
      if (mounted) {
        setState(() {});
      }
      if (photoController!.value.hasError) {
        showInSnackBar('Camera error ${photoController!.value.errorDescription}');
      }
    });

    try {
      await photoController!.initialize();
      await Future.wait(<Future<Object?>>[
        ...!kIsWeb ? <Future<Object?>>[photoController!.getMinExposureOffset().then((double value) => _minAvailableExposureOffset = value), photoController!.getMaxExposureOffset().then((double value) => _maxAvailableExposureOffset = value)] : <Future<Object?>>[],
        photoController!.getMaxZoomLevel().then((double value) => _maxAvailableZoom = value),
        photoController!.getMinZoomLevel().then((double value) => _minAvailableZoom = value),
      ]);
    } on CameraException catch (e) {
      switch (e.code) {
        case 'CameraAccessDenied':
          showInSnackBar('Non hai gli accessi alla telecamera.');
          break;
        case 'CameraAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi alla Telecamera.");
          break;
        case 'CameraAccessRestricted':
          // iOS only
          showInSnackBar('Accesso riservato per la Telecamera');
          break;
        case 'AudioAccessDenied':
          showInSnackBar("Non hai gli accessi per registrare l'audio");
          break;
        case 'AudioAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi all'Audio.");
          break;
        case 'AudioAccessRestricted':
          // iOS only
          showInSnackBar("Accesso riservato per la registrazione Audio");
          break;
        default:
          _showCameraException(e);
          break;
      }
    }

    if (mounted) {
      setState(() {});
    }
  }

  void _handleScaleStart(ScaleStartDetails details) {
    _baseScale = _currentScale;
  }

  Future<void> _handleScaleUpdate(ScaleUpdateDetails details) async {
    if (pointers != 2) {
      return;
    }
    _currentScale = (_baseScale * details.scale).clamp(_minAvailableZoom, _maxAvailableZoom);
    await photoController!.setZoomLevel(_currentScale);
  }

  Widget _cameraPreviewWidget() {
    // return photoController!.buildPreview();

    return CameraPreview(photoController!, child: LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        onScaleStart: _handleScaleStart,
        onScaleUpdate: _handleScaleUpdate,
        onTapDown: (TapDownDetails details) => onViewFinderTap(details, constraints),
      );
    }));
  }

  void onViewFinderTap(TapDownDetails details, BoxConstraints constraints) {
    if (photoController == null) {
      return;
    }

    final CameraController cameraController = photoController!;

    final Offset offset = Offset(
      details.localPosition.dx / constraints.maxWidth,
      details.localPosition.dy / constraints.maxHeight,
    );
    cameraController.setExposurePoint(offset);
    cameraController.setFocusPoint(offset);
  }

  Future<XFile?> takePicture() async {
    // photoController!.setFocusMode(FocusMode.locked);
    final CameraController? cameraController = photoController!;
    if (cameraController == null || !cameraController.value.isInitialized) {
      return null;
    }
    if (cameraController.value.isTakingPicture) {
      return null;
    }
    try {
      cameraController.setFocusMode(FocusMode.locked);
      // cameraController.setFocusPoint(point)
      var r = await cameraController.takePicture();
      return r;
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  void _showCameraException(CameraException e) {
    // print(e.description);
    showInSnackBar('Error: ${e.code}\n${e.description}');
  }

  void onTakePictureButtonPressed(T currentDataItem, void Function(void Function()) setState) async {
    //await photoController!.pausePreview();

    setState(() {
      waiterMessage = "Sto salvando la Foto...";
      widget.onTakeMedia!(setState);
    });
    var id = Uuid().v4();
    takePicture().then((XFile? file) async {
      //await photoController!.pausePreview();
      if (mounted) {
        imageFile = file;

        if (file != null) {
          // widget.onTake_Media((currentDataItem as XFDataItem).id);
          var x = XMediaItem(id, xDocID: (currentDataItem as XFDataItem).id, mediaType: XEnums.mediaType.image.value, payload: await file.readAsBytes(), isNew: true);
          widget.lXMedias.add(x);
          widget.parentSetState(() {});
          // var fileTmp = File("${XLibStorage.filePath}/IMMAGINI/${id}_.jpg");
          // fileTmp.createSync(recursive: true);
          // await widget.onTake_Picture(file, currentDataItem, isBusy, setState);
        }
      }
    }).whenComplete(() async {
      setState(() {
        waiterMessage = "";
        widget.onTakeMedia!(setState);
      });
      //await photoController!.resumePreview();
    });
  }

  onVideoRecordButtonPressed(T lav, void Function(void Function()) setState) async {
    setState(() {
      waiterMessage = "Sta per cominciare la Registrazione...";
      widget.onStartTakeVideo!(setState);
    });
    await startVideoRecording(lav, setState).then((_) {
      if (mounted) {
        setState(() {
          waiterMessage = "";
        });
      }
    });
  }

  Widget _captureControlRowWidget(T lav, void Function(void Function()) setState) {
    return Container(
      width: orientation == Orientation.portrait ? MediaQuery.of(context).size.width : 60,
      height: orientation == Orientation.portrait ? 60 : null,
      margin: EdgeInsets.symmetric(vertical: 4),
      child: orientation == Orientation.portrait ? Row(mainAxisAlignment: MainAxisAlignment.end, children: liControl_BTN(lav, setState)) : Column(mainAxisAlignment: MainAxisAlignment.end, children: liControl_BTN(lav, setState)),

      // Row(mainAxisAlignment: MainAxisAlignment.end, children: [
      //   Row(
      //     mainAxisSize: MainAxisSize.min,
      //     mainAxisAlignment: MainAxisAlignment.end,
      //     children: [
      //       SizedBox(width: isRecording_Audio.not() ? 0 : 10),
      //       isRecording_Audio.not()
      //           ? Container()
      //           : Container(
      //               decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
      //               child: IconButton(
      //                 icon: Icon(Icons.stop),
      //                 color: Colors.red,
      //                 onPressed: () async => await _startOrStopRecording(lav as XFDataItem),
      //               )),
      //     ],
      //   ),
      // ]),
    );
  }

  liControl_BTN(T lav, void Function(void Function()) setState) {
    Color color_BackGroundBTNactions = Color.fromARGB(255, 0, 45, 97);
    return [
      Container(
          decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
          child: IconButton(
            icon: Icon(Icons.flash_on_outlined),
            color: photoController!.value.flashMode == FlashMode.off
                ? Colors.white
                : photoController!.value.isInitialized
                    ? Colors.yellow
                    : Colors.white,
            onPressed: photoController!.value.isInitialized
                ? photoController!.value.flashMode == FlashMode.off
                    ? () => setFlashMode(FlashMode.torch, setState)
                    : () => setFlashMode(FlashMode.off, setState)
                : null,
          )),
      SizedBox(width: 4),
      photoController!.value.isRecordingVideo || isRecording_Audio
          ? Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
              child: IconButton(
                icon: Icon(Icons.delete_outline),
                color: Colors.red,
                onPressed: () async {
                  if (isRecording_Audio) {
                    recorderController.stop();
                    setState(() {
                      isRecording_Audio = false;
                    });
                  } else {
                    await photoController!.stopVideoRecording();
                  }
                  widget.onEndTakeVideo!(setState);
                },
              ))
          : Container(),
      SizedBox(width: 4),
      isRecording_Audio
          ? Container()
          : photoController!.value.isRecordingVideo.not()
              ? Container()
              : Container(
                  decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                  child: IconButton(
                    icon: photoController!.value.isRecordingPaused ? Icon(Icons.play_arrow) : Icon(Icons.pause),
                    color: Colors.white,
                    onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo
                        ? (photoController!.value.isRecordingPaused)
                            ? () => onResumeButtonPressed(setState)
                            : () => onPauseButtonPressed(setState)
                        : null,
                  )),
      SizedBox(width: 4),
      isRecording_Audio
          ? Container()
          : photoController!.value.isRecordingVideo.not()
              ? Container()
              : Container(
                  decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                  child: IconButton(
                    icon: Icon(Icons.stop),
                    color: Colors.red,
                    onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo ? () async => await onStopButtonPressed(lav, setState) : null,
                  )),
      // SizedBox(width: 4),
      // photoController!.value.isRecordingVideo || isRecording_Audio
      //     ? Container()
      //     : Container(
      //         decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
      //         child: IconButton(
      //           icon: Icon(Icons.video_camera_back_outlined),
      //           color: outCamera ? Colors.green : Colors.white,
      //           onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => cmdViewAnotherCamera() : null,
      //         )),
      SizedBox(width: 4),
      photoController!.value.isRecordingVideo
          ? Container()
          : Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
              child: IconButton(
                icon: Icon(isRecording_Audio.not() ? Icons.mic : Icons.stop),
                color: isRecording_Audio.not() ? Colors.blue : Colors.red,
                onPressed: () async => await _startOrStopRecording_Audio(lav as XFDataItem),
              )),
      SizedBox(width: 4),
      isRecording_Audio
          ? Container()
          : Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
              child: IconButton(
                icon: Icon(Icons.videocam),
                color: Colors.white,
                onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () async => await onVideoRecordButtonPressed(lav, setState) : null,
              )),
      SizedBox(width: 4),
      photoController!.value.isRecordingVideo || isRecording_Audio
          ? Container()
          : Container(
              decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
              child: IconButton(
                icon: Icon(Icons.camera_alt),
                color: Colors.white,
                onPressed: photoController!.value.isInitialized ? () => onTakePictureButtonPressed(lav, setState) : null,
              )),
    ];
  }

  void _getDir() async {
    // appDirectory = await getApplicationDocumentsDirectory();
    // path = "${appDirectory.path}/recording.m4a";
    isLoading = false;
    setState(() {});
  }

  void _initialiseControllers() {
    recorderController = RecorderController()
      ..androidEncoder = AndroidEncoder.aac
      ..androidOutputFormat = AndroidOutputFormat.mpeg4
      ..iosEncoder = IosEncoder.kAudioFormatMPEG4AAC
      ..sampleRate = 44100;
  }

  var idForAudio;
  Future _startOrStopRecording_Audio(XFDataItem item) async {
    widget.onStartTakeVideo!(setState);

    try {
      if (isRecording_Audio) {
        recorderController.reset();
        final path = await recorderController.stop(false);
        if (path != null) {
          isRecordingCompleted = true;
          debugPrint(path);
          debugPrint("Recorded file size: ${File(path).lengthSync()}");
          File(path).readAsBytes().then((value) async {
            var xclMedia = XMediaItem(idForAudio, xDocID: item.id, localURI: xFileUI!.path, payload: value, mediaType: XEnums.mediaType.audio.value, isNew: true);
            widget.parentSetState(() {
              widget.lXMedias.add(xclMedia);
            });
            await widget.onEndTakeVideo!(setState);
          });
        }
      } else {
        idForAudio = Uuid().v4();
        xFileUI = File("${XLibStorage.filePath}/XMEDIAS/${idForAudio}_.aac");
        await xFileUI!.create(recursive: true).then((value) async => await recorderController.record(path: xFileUI!.path));
        widget.onStartTakeVideo!(setState);
      }
    } catch (e) {
      debugPrint(e.toString());
    } finally {
      setState(() {
        isRecording_Audio = !isRecording_Audio;
      });
    }
  }

  cmdViewAnotherCamera() async {
    if (outCamera.not()) {
      var x = await showDialog(
          context: context,
          builder: (context) {
            return XAlertDialog(
                title_Text: "Inserisci URL rtsp:",
                child: Column(mainAxisSize: MainAxisSize.min, children: [
                  Row(children: [Expanded(child: XTextFormField(controller: urlRTSPcontroller, height: null))])
                ]),
                actionsBTNarea_Child: Row(children: [
                  Expanded(
                      child: XBtnbase(
                          margin: EdgeInsets.symmetric(horizontal: 10),
                          label: "Annulla",
                          onPressed: () {
                            urlRTSP = "";
                            Navigator.pop(context, false);
                          },
                          label_Color: Colors.red)),
                  Expanded(
                      child: XBtnbase(
                          margin: EdgeInsets.symmetric(horizontal: 10),
                          label: "Salva",
                          onPressed: () {
                            if (urlRTSPcontroller!.text != urlRTSP) {
                              urlRTSP = urlRTSPcontroller!.text;
                            } else {}
                            outCamera = outCamera.not();
                            Navigator.pop(context, true);
                          },
                          label_Color: Colors.green))
                ]));
          });
      setState(() {
        widget.onTakeMedia!(setState);
      });
      if (x != null && x) {
        // player.setOption(FijkOption.playerCategory, "mediacodec-all-videos", 1);
        // player = FijkPlayer();
        if (urlRTSP != null) {
          // await player.setDataSource(urlRTSP!, autoPlay: true); //'rtsp://admin:ldmospiti@192.168.1.120:554/live/CAM2'
        }
        // player.isPlayable();
      } else {
        outCamera = false;
      }

      setState(() {
        widget.onTakeMedia!(setState);
      });
    } else {
      setState(() {
        outCamera = outCamera.not();
      });
    }
  }

  onStopButtonPressed(T lav, void Function(void Function()) setState) async {
    setState(() {
      waiterMessage = "Sto salvando la Registrazione...";
      widget.onEndTakeVideo!(setState);
    });
    await stopVideoRecording(lav, setState).then((XFile? file) {
      if (mounted) {
        setState(() {});
      }
      if (file != null) {
        videoFile = file;
      }
    });
  }

  Future<void> onPausePreviewButtonPressed(void Function(void Function()) setState) async {
    final CameraController? cameraController = photoController!;

    if (cameraController == null || !cameraController.value.isInitialized) {
      // showInSnackBar('Error: select a camera first.');
      return;
    }
    if (cameraController.value.isPreviewPaused) {
      await cameraController.resumePreview();
    } else {
      await cameraController.pausePreview();
    }

    if (mounted) {
      setState(() {});
    }
  }

  void onPauseButtonPressed(void Function(void Function()) setState) {
    pauseVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video messo in Pausa');
    });
  }

  void onResumeButtonPressed(void Function(void Function()) setState) {
    resumeVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video ripartito');
    });
  }

  Future<void> startVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isInitialized) {
      showInSnackBar('Error: Prima seleziona un telecamera.');
      return;
    }

    if (photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.startVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return;
    }
  }

  Future<Uint8List?> generateVideoThumbnail(String path) async {
    var x = await thumb.VideoThumbnail.thumbnailData(
      video: path,
      imageFormat: thumb.ImageFormat.PNG,
      maxWidth: 128, // specify the width of the thumbnail, let the height auto-scaled to keep the source aspect ratio
      quality: 25,
    );
    return x;
  }

  Future<XFile?> stopVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return null;
    }

    try {
      await photoController!.stopVideoRecording().then((filexFromCam) async {
        var id = Uuid().v4();
        var fileTmp = File("${XLibStorage.filePath}/XMEDIAS/${id}_.mp4");
        fileTmp.createSync(recursive: true);
        var minia = await generateVideoThumbnail(filexFromCam.path);
        await filexFromCam.readAsBytes().then((bytes) {
          var xm = XMediaItem(id, payload: bytes, xDocID: (lav as XFDataItem).id, miniatura: minia, mediaType: XEnums.mediaType.video.value, isNew: true);
          widget.parentSetState(() {
            widget.lXMedias.add(xm);
          });

          videoFile = filexFromCam;
          // widget.onTake_Video(videoFile!, lav, onTakeMedia, setState);
          //    fileTmp.delete();
        });
        setState(() {
          waiterMessage = "";
        });
        return filexFromCam;
      });
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
    return null;
  }

  Future<void> pauseVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.pauseVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<void> resumeVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.resumeVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<bool> setFlashMode(FlashMode mode, void Function(void Function()) setState) async {
    if (photoController == null) {
      return false;
    }

    try {
      await photoController!.setFlashMode(mode);
      if (photoController!.value.flashMode == FlashMode.off) {
        setState(() {});
        return false;
      } else {
        setState(() {});
        return true;
      }
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  // Widget _thumbnailWidget() {
  //   final VideoPlayerController? localVideoController = videoController;

  //   return Expanded(
  //       child: Align(
  //           alignment: Alignment.centerRight,
  //           child: Row(mainAxisSize: MainAxisSize.min, children: <Widget>[
  //             if (localVideoController == null && imageFile == null)
  //               Container()
  //             else
  //               SizedBox(
  //                   width: 64.0,
  //                   height: 64.0,
  //                   child: (localVideoController == null)
  //                       ? (kIsWeb ? Image.network(imageFile!.path) : Image.file(File(imageFile!.path)))
  //                       : Container(
  //                           decoration: BoxDecoration(border: Border.all(color: Colors.pink)),
  //                           child: Center(
  //                             child: AspectRatio(aspectRatio: localVideoController.value.size != null ? localVideoController.value.aspectRatio : 1.0, child: VideoPlayer(localVideoController)),
  //                           )))
  //           ])));
  // }
  @override
  Widget build(BuildContext context) {
    orientation = MediaQuery.of(context).orientation;
    return (photoController != null && photoController!.value.isInitialized)
        ? Container(
            decoration: BoxDecoration(color: Colors.black, border: Border.all(width: 3.0, color: photoController!.value.isRecordingVideo ? Colors.redAccent : Colors.grey)),
            child: orientation == Orientation.portrait
                ? Column(
                    mainAxisSize: MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      ///Widget per la registrazione degli audio
                      isRecording_Audio
                          ? Container(
                              child: AudioWaveforms(
                              size: Size(MediaQuery.of(context).size.width, 200.0),
                              recorderController: recorderController,
                              enableGesture: true,
                              waveStyle: WaveStyle(
                                showDurationLabel: true,
                                spacing: 8.0,
                                showBottom: false,
                                extendWaveform: true,
                                showMiddleLine: false,
                              ),
                            ))

                          ///Widget per fare video e foto
                          : Expanded(
                              child: Stack(alignment: Alignment.center, children: [
                              _cameraPreviewWidget(),
                              //  Container(color: Colors.black.withAlpha(90)) : Container(),
                              // widget.isBusy ? XStatefulWidgetBase.LoaderIndicator(widget.isBusy, label: waiterMessage) : Container(),
                            ])),

                      ///Comandi per la gestone dei media
                      _captureControlRowWidget(widget.itemOriginal, setState),
                    ],
                  )
                : Row(
                    children: [
                      ///Widget per la registrazione degli audio
                      isRecording_Audio
                          ? Container(
                              child: AudioWaveforms(
                              size: Size(MediaQuery.of(context).size.width, 200.0),
                              recorderController: recorderController,
                              enableGesture: true,
                              waveStyle: WaveStyle(
                                showDurationLabel: true,
                                spacing: 8.0,
                                showBottom: false,
                                extendWaveform: true,
                                showMiddleLine: false,
                              ),
                            ))

                          ///Widget per fare video e foto
                          : Expanded(
                              child: Stack(alignment: Alignment.center, children: [
                              _cameraPreviewWidget(),
                              // widget.isBusy ? Container(color: Colors.black.withAlpha(90)) : Container(),
                              // widget.isBusy ? XStatefulWidgetBase.LoaderIndicator(isBusy, label: waiterMessage) : Container(),
                            ])),

                      ///Comandi per la gestone dei media
                      _captureControlRowWidget(widget.itemOriginal, setState),
                    ],
                  ),
          )
        : XStatefulWidgetBase.LoaderIndicator(true);
  }
}
